@@ -34,16 +34,26 @@
 #include "peas-extension.h"
 #include "peas-dirs.h"
 #include "peas-debug.h"
+#include "peas-helpers.h"
 
 /**
  * SECTION:peas-engine
  * @short_description: Engine at the heart of the Peas plugin system.
  * @see_also: #PeasPluginInfo
  *
- * The #PeasEngine is the object which manages the plugins.  Its role is twofold:
- * First it will fetch all the information about the available plugins from all
- * the registered plugin directories, and second it will provide you an API to
- * load, control and unload the plugins from within your application.
+ * The #PeasEngine is the object which manages the plugins.
+ *
+ * Its role is twofold:
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>it will fetch all the information about the available plugins
+ *     from all the registered plugin directories;</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>it will provide you an API to load, control and unload your
+ *     plugins and their extensions from within your application.</para>
+ *   </listitem>
+ * </itemizedlist>
  **/
 G_DEFINE_TYPE (PeasEngine, peas_engine, G_TYPE_OBJECT);
 
@@ -449,9 +459,9 @@ peas_engine_class_init (PeasEngineClass *klass)
    * when it comes to installation location: the same plugin can be
    * installed both in the system path or in the user's home directory,
    * without taking other special care than using
-   * egg_plugin_get_data_dir() when looking for its data files.
+   * peas_plugin_info_get_data_dir() when looking for its data files.
    *
-   * Concretely, this property contains a NULL-terminated array of
+   * Concretely, this property contains a %NULL-terminated array of
    * strings, whose even-indexed strings are module directories, and
    * odd-indexed ones are the associated data directories.  Here is an
    * example of such an array:
@@ -528,6 +538,12 @@ peas_engine_class_init (PeasEngineClass *klass)
    * @info: A #PeasPluginInfo.
    *
    * The load-plugin signal is emitted when a plugin is being loaded.
+   *
+   * The plugin is being loaded in the default handler. Hence, if you want to
+   * perform some action before the plugin is loaded, you should use
+   * g_signal_connect(), but if you want to perform some action *after* the
+   * plugin is loaded (the most common case), you should use
+   * g_signal_connect_after().
    */
   signals[LOAD_PLUGIN] =
     g_signal_new ("load-plugin",
@@ -547,6 +563,12 @@ peas_engine_class_init (PeasEngineClass *klass)
    * @info: A #PeasPluginInfo.
    *
    * The unload-plugin signal is emitted when a plugin is being unloaded.
+   *
+   * The plugin is being unloaded in the default handler. Hence, if you want
+   * to perform some action before the plugin is unloaded (the most common
+   * case), you should use g_signal_connect(), but if you want to perform some
+   * action after the plugin is unloaded (the most common case), you should
+   * use g_signal_connect_after().
    */
   signals[UNLOAD_PLUGIN] =
     g_signal_new ("unload-plugin",
@@ -753,6 +775,9 @@ static gboolean
 load_plugin (PeasEngine     *engine,
              PeasPluginInfo *info)
 {
+  const gchar **dependencies;
+  PeasPluginInfo *dep_info;
+  guint i;
   PeasPluginLoader *loader;
 
   if (peas_plugin_info_is_loaded (info))
@@ -761,6 +786,24 @@ load_plugin (PeasEngine     *engine,
   if (!peas_plugin_info_is_available (info))
     return FALSE;
 
+  /* We set the plugin info as loaded before trying to load the dependencies,
+   * to make sure we won't have an infinite loop. */
+  info->loaded = TRUE;
+
+  dependencies = peas_plugin_info_get_dependencies (info);
+  for (i = 0; dependencies[i] != NULL; i++)
+    {
+      dep_info = peas_engine_get_plugin_info (engine, dependencies[i]);
+      if (!dep_info)
+        {
+          g_warning ("Plugin not found: %s", dependencies[i]);
+          return FALSE;
+        }
+
+      if (!peas_engine_load_plugin (engine, dep_info))
+        return FALSE;
+    }
+
   loader = get_plugin_loader (engine, info);
 
   if (loader == NULL)
@@ -771,11 +814,10 @@ load_plugin (PeasEngine     *engine,
       return FALSE;
     }
 
-  info->loaded = peas_plugin_loader_load (loader, info);
-
-  if (info->loaded == FALSE)
+  if (!peas_plugin_loader_load (loader, info))
     {
       g_warning ("Error loading plugin '%s'", info->name);
+      info->loaded = FALSE;
       info->available = FALSE;
       return FALSE;
     }
@@ -797,8 +839,10 @@ peas_engine_load_plugin_real (PeasEngine     *engine,
  * @info: A #PeasPluginInfo.
  *
  * Loads the plugin corresponding to @info if it's not currently loaded.
+ * Emits the "load-plugin" signal; loading the plugin
+ * actually occurs in the default signal handler.
  *
- * Returns: whether the plugin has been successfuly loaded.
+ * Returns: whether the plugin has been successfully loaded.
  */
 gboolean
 peas_engine_load_plugin (PeasEngine     *engine,
@@ -821,20 +865,37 @@ static void
 peas_engine_unload_plugin_real (PeasEngine     *engine,
                                 PeasPluginInfo *info)
 {
+  GList *item;
+  const gchar *module_name;
   PeasPluginLoader *loader;
 
   if (!peas_plugin_info_is_loaded (info) ||
       !peas_plugin_info_is_available (info))
     return;
 
+  /* We set the plugin info as unloaded before trying to unload the
+   * dependants, to make sure we won't have an infinite loop. */
+  info->loaded = FALSE;
+
+  /* First unload all the dependant plugins */
+  module_name = peas_plugin_info_get_module_name (info);
+  for (item = engine->priv->plugin_list; item; item = item->next)
+    {
+      PeasPluginInfo *other_info = PEAS_PLUGIN_INFO (item->data);
+
+      if (!peas_plugin_info_is_loaded (other_info))
+        continue;
+
+      if (peas_plugin_info_has_dependency (other_info, module_name))
+         peas_engine_unload_plugin (engine, other_info);
+    }
+
   /* find the loader and tell it to gc and unload the plugin */
   loader = get_plugin_loader (engine, info);
 
   peas_plugin_loader_garbage_collect (loader);
   peas_plugin_loader_unload (loader, info);
 
-  info->loaded = FALSE;
-
   g_object_notify (G_OBJECT (engine), "loaded-plugins");
 }
 
@@ -844,8 +905,10 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
  * @info: A #PeasPluginInfo.
  *
  * Unloads the plugin corresponding to @info.
+ * Emits the "unload-plugin" signal; unloading the plugin
+ * actually occurs in the default signal handler.
  *
- * Returns: whether the plugin has been successfuly unloaded.
+ * Returns: whether the plugin has been successfully unloaded.
  */
 gboolean
 peas_engine_unload_plugin (PeasEngine     *engine,
@@ -878,10 +941,29 @@ peas_engine_provides_extension (PeasEngine     *engine,
   return peas_plugin_loader_provides_extension (loader, info, extension_type);
 }
 
+/**
+ * peas_engine_create_extensionv
+ * @engine: A #PeasEngine.
+ * @info: A loaded #PeasPluginInfo.
+ * @extension_type: The implemented extension #GType.
+ * @n_parameters: the length of the @parameters array.
+ * @parameters: an array of #GParameter.
+ *
+ * If the plugin identified by @info implements the @extension_type interface,
+ * then this function will return a new instance of this implementation,
+ * wrapped in a new #PeasExtension instance. Otherwise, it will return %NULL.
+ *
+ * See peas_engine_create_extension() for more information.
+ *
+ * Returns: a new instance of #PeasExtension wrapping the @extension_type
+ * instance, or %NULL.
+ */
 PeasExtension *
-peas_engine_get_extension (PeasEngine     *engine,
-                           PeasPluginInfo *info,
-                           GType           extension_type)
+peas_engine_create_extensionv (PeasEngine     *engine,
+                               PeasPluginInfo *info,
+                               GType           extension_type,
+                               guint           n_parameters,
+                               GParameter     *parameters)
 {
   PeasPluginLoader *loader;
 
@@ -889,16 +971,112 @@ peas_engine_get_extension (PeasEngine     *engine,
   g_return_val_if_fail (info != NULL, NULL);
 
   loader = get_plugin_loader (engine, info);
-  return peas_plugin_loader_get_extension (loader, info, extension_type);
+  return peas_plugin_loader_create_extension (loader, info, extension_type,
+                                              n_parameters, parameters);
+}
+
+/**
+ * peas_engine_create_extension_valist
+ * @engine: A #PeasEngine.
+ * @info: A loaded #PeasPluginInfo.
+ * @extension_type: The implemented extension #GType.
+ * @first_property: the name of the first property.
+ * @var_args: the value of the first property, followed optionally by more
+ *   name/value pairs, followed by %NULL.
+ *
+ * If the plugin identified by @info implements the @extension_type interface,
+ * then this function will return a new instance of this implementation,
+ * wrapped in a new #PeasExtension instance. Otherwise, it will return %NULL.
+ *
+ * See peas_engine_create_extension() for more information.
+ *
+ * Returns: a new instance of #PeasExtension wrapping the @extension_type
+ * instance, or %NULL.
+ */
+PeasExtension *
+peas_engine_create_extension_valist (PeasEngine     *engine,
+                                     PeasPluginInfo *info,
+                                     GType           extension_type,
+                                     const gchar    *first_property,
+                                     va_list         var_args)
+{
+  gpointer type_struct;
+  guint n_parameters;
+  GParameter *parameters;
+  PeasExtension *exten;
+
+  type_struct = _g_type_struct_ref (extension_type);
+
+  if (!_valist_to_parameter_list (extension_type, type_struct, first_property,
+                                  var_args, &parameters, &n_parameters))
+    {
+      g_return_val_if_reached (NULL);
+    }
+
+  exten = peas_engine_create_extensionv (engine, info, extension_type,
+                                         n_parameters, parameters);
+
+  while (n_parameters-- > 0)
+    g_value_unset (&parameters[n_parameters].value);
+  g_free (parameters);
+
+  _g_type_struct_unref (extension_type, type_struct);
+
+  return exten;
+}
+
+/**
+ * peas_engine_create_extension:
+ * @engine: A #PeasEngine.
+ * @info: A loaded #PeasPluginInfo.
+ * @extension_type: The implemented extension #GType.
+ * @first_property: the name of the first property.
+ * @Varargs: the value of the first property, followed optionally by more
+ *   name/value pairs, followed by %NULL.
+ *
+ * If the plugin identified by @info implements the @extension_type interface,
+ * then this function will return a new instance of this implementation,
+ * wrapped in a new #PeasExtension instance. Otherwise, it will return %NULL.
+ *
+ * When creating the new instance of the @extension_type subtype, the
+ * provided construct properties will be passed to the extension construction
+ * handler (exactly like if you had called g_object_new() yourself).
+ *
+ * The new extension instance produced by this function will always be
+ * returned wrapped in a #PeasExtension proxy, following the current libpeas
+ * principle of never giving you the actual object (also because it might as
+ * well *not* be an actual object).
+ *
+ * Returns: a new instance of #PeasExtension wrapping the @extension_type
+ * instance, or %NULL.
+ */
+PeasExtension *
+peas_engine_create_extension (PeasEngine     *engine,
+                              PeasPluginInfo *info,
+                              GType           extension_type,
+                              const gchar    *first_property,
+                              ...)
+{
+  va_list var_args;
+  PeasExtension *exten;
+
+  va_start (var_args, first_property);
+  exten = peas_engine_create_extension_valist (engine, info, extension_type,
+                                               first_property, var_args);
+  va_end (var_args);
+
+  return exten;
 }
 
 /**
  * peas_engine_get_loaded_plugins:
  * @engine: A #PeasEngine.
  *
- * Returns the list of the names of all the loaded plugins, or %NULL if there
- * is no plugin currently loaded. Please note that the returned array is a
- * newly allocated one: you will need to free it using g_strfreev().
+ * Returns the list of the names of all the loaded plugins, or an array
+ * containing a single %NULL element if there is no plugin currently loaded.
+ *
+ * Please note that the returned array is a newly allocated one: you will need
+ * to free it using g_strfreev().
  *
  * Returns: A newly-allocated NULL-terminated array of strings, or %NULL.
  */
