@@ -37,14 +37,28 @@ G_BEGIN_DECLS
 #define PEAS_IS_OBJECT_MODULE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), PEAS_TYPE_OBJECT_MODULE))
 #define PEAS_OBJECT_MODULE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS((obj), PEAS_TYPE_OBJECT_MODULE, PeasObjectModuleClass))
 
-typedef struct _PeasObjectModule        PeasObjectModule;
-typedef struct _PeasObjectModuleClass   PeasObjectModuleClass;
-typedef struct _PeasObjectModulePrivate PeasObjectModulePrivate;
+typedef struct _PeasObjectModule         PeasObjectModule;
+typedef struct _PeasObjectModuleClass    PeasObjectModuleClass;
+typedef struct _PeasObjectModulePrivate  PeasObjectModulePrivate;
 
+/**
+ * PeasFactoryFunc:
+ *
+ * A #PeasFactoryFunc is a factory function which will instanciate a new
+ * extension of a given type. g_object_newv() is such a function.
+ *
+ * It is used with peas_object_module_register_object_factory().
+ */
 typedef GObject *(*PeasFactoryFunc)   (guint          n_parameters,
                                        GParameter    *parameters,
                                        gpointer       user_data);
 
+/**
+ * PeasObjectModule:
+ *
+ * The #PeasObjectModule structure contains only private data and should only
+ * be accessed using the provided API.
+ */
 struct _PeasObjectModule {
   GTypeModule parent;
 
@@ -60,7 +74,6 @@ PeasObjectModule   *peas_object_module_new                    (const gchar
                                                                const gchar      *path,
                                                                gboolean          resident);
 
-void                peas_object_module_register_types         (PeasObjectModule *module);
 GObject            *peas_object_module_create_object          (PeasObjectModule *module,
                                                                GType             interface,
                                                                guint             n_parameters,
