@@ -5,18 +5,18 @@ var LABEL_STRING = "Seed Says Hello Too!";
 print("LABEL_STRING=" +  LABEL_STRING);
 
 activatable_extension = {
-  activate: function(win) {
+  activate: function() {
     print("SeedHelloPlugin.activate");
-    win._seedhello_label = new Gtk.Label({ label: LABEL_STRING });
-    win._seedhello_label.show();
-    win.get_child().pack_start(win._seedhello_label);
+    this.object._seedhello_label = new Gtk.Label({ label: LABEL_STRING });
+    this.object._seedhello_label.show();
+    this.object.get_child().pack_start(this.object._seedhello_label);
   },
-  deactivate: function(win) {
+  deactivate: function() {
     print("SeedHelloPlugin.deactivate");
-    win.get_child().remove(win._seedhello_label);
-    win._seedhello_label.destroy();
+    this.object.get_child().remove(this.object._seedhello_label);
+    this.object._seedhello_label.destroy();
   },
-  update_state: function(win) {
+  update_state: function() {
     print("SeedHelloPlugin.update_state");
   }
 };
